@@ -98,6 +98,20 @@ import org.w3c.dom.Text;
  */
 public class Reference extends SignatureElementProxy {
 
+   /**
+    * Look up useC14N11 system property. If true, an explicit C14N11 transform
+    * will be added if necessary when generating the signature. See section
+    * 3.1.1 of http://www.w3.org/2007/xmlsec/Drafts/xmldsig-core/ for more info.
+    */
+   private static boolean useC14N11 = false;
+   static {
+      try {
+         useC14N11 = Boolean.getBoolean("org.apache.xml.security.useC14N11");
+      } catch (Exception e) {
+         // ignore exceptions
+      }
+   }
+
    /** Field CacheSignedNodes */
    public final static boolean CacheSignedNodes = false;
 
@@ -233,7 +247,7 @@ private Element digestValueElement;
     */
    public void setURI(String URI) {
 
-      if ((this._state == MODE_SIGN) && (URI != null)) {
+      if ( URI != null) {
          this._constructionElement.setAttributeNS(null, Constants._ATT_URI,
                                                   URI);
       }
@@ -255,7 +269,7 @@ private Element digestValueElement;
     */
    public void setId(String Id) {
 
-      if ((this._state == MODE_SIGN) && (Id != null)) {
+      if ( Id != null ) {
          this._constructionElement.setAttributeNS(null, Constants._ATT_ID, Id);
          IdResolver.registerElementById(this._constructionElement, Id);
       }
@@ -277,7 +291,7 @@ private Element digestValueElement;
     */
    public void setType(String Type) {
 
-      if ((this._state == MODE_SIGN) && (Type != null)) {
+      if (Type != null) {
          this._constructionElement.setAttributeNS(null, Constants._ATT_TYPE,
                                                   Type);
       }
@@ -334,9 +348,6 @@ private Element digestValueElement;
     */
    private void setDigestValueElement(byte[] digestValue)
    {
-
-      if (this._state == MODE_SIGN) {
-
          Node n=digestValueElement.getFirstChild();
          while (n!=null) {
                digestValueElement.removeChild(n);
@@ -347,7 +358,6 @@ private Element digestValueElement;
          Text t = this._doc.createTextNode(base64codedValue);
 
          digestValueElement.appendChild(t);
-      }
    }
 
    /**
@@ -358,11 +368,7 @@ private Element digestValueElement;
     */
    public void generateDigestValue()
            throws XMLSignatureException, ReferenceNotInitializedException {
-
-      if (this._state == MODE_SIGN) {
-
-         this.setDigestValueElement(this.calculateDigest());
-      }
+      this.setDigestValueElement(this.calculateDigest(false));
    }
 
    /**
@@ -670,13 +676,14 @@ private Element digestValueElement;
 
 
    /**
-    * Method resolverResult
+    * Method calculateDigest
     *
+    * @param validating true if validating the reference
     * @return reference Calculate the digest of this reference.
     * @throws ReferenceNotInitializedException
     * @throws XMLSignatureException
     */
-   private byte[] calculateDigest()
+   private byte[] calculateDigest(boolean validating)
            throws ReferenceNotInitializedException, XMLSignatureException {
 
       try {
@@ -687,7 +694,20 @@ private Element digestValueElement;
          DigesterOutputStream diOs=new DigesterOutputStream(mda);
          OutputStream os=new UnsyncBufferedOutputStream(diOs);
          XMLSignatureInput output=this.dereferenceURIandPerformTransforms(os);         
-         output.updateOutputStream(os);
+	 // if signing and c14n11 property == true explicitly add
+	 // C14N11 transform if needed
+	 if (this.useC14N11 && !validating &&
+	     !output.isOutputStreamSet() && !output.isOctetStream()) {
+	     if (transforms == null) {
+		 transforms = new Transforms(this._doc);
+                 this._constructionElement.insertBefore
+		     (transforms.getElement(), digestMethodElem);
+	     }
+	     transforms.addTransform(Transforms.TRANSFORM_C14N11_OMIT_COMMENTS);
+             output.updateOutputStream(os, true);
+	 } else {
+             output.updateOutputStream(os);
+	 }
          os.flush();
          //this.getReferencedBytes(diOs);
          //mda.update(data);
@@ -697,7 +717,7 @@ private Element digestValueElement;
          throw new ReferenceNotInitializedException("empty", ex);
       } catch (IOException ex) {
       	 throw new ReferenceNotInitializedException("empty", ex);
-	}
+      }
    }
 
    /**
@@ -705,7 +725,7 @@ private Element digestValueElement;
     *
     * @return the digest value.
     * @throws Base64DecodingException if Reference contains no proper base64 encoded data.
-	* @throws XMLSecurityException if the Reference does not contain a DigestValue element
+    * @throws XMLSecurityException if the Reference does not contain a DigestValue element
     */
    public byte[] getDigestValue() throws Base64DecodingException, XMLSecurityException {
       if (digestValueElement == null) {
@@ -732,11 +752,13 @@ private Element digestValueElement;
            throws ReferenceNotInitializedException, XMLSecurityException {
 
       byte[] elemDig = this.getDigestValue();
-      byte[] calcDig = this.calculateDigest();
+      byte[] calcDig = this.calculateDigest(true);
       boolean equal = MessageDigestAlgorithm.isEqual(elemDig, calcDig);
 
       if (!equal) {
          log.warn("Verification failed for URI \"" + this.getURI() + "\"");
+         log.warn("Expected Digest: " + Base64.encode(elemDig));
+         log.warn("Actual Digest: " + Base64.encode(calcDig));
       } else {
          log.info("Verification successful for URI \"" + this.getURI() + "\"");
       }
