@@ -71,7 +71,7 @@ import org.w3c.dom.Text;
  * <li>sign and checkSignatureValue methods are used to sign and validate the
  * signature. </li></ul>
  *
- * @author $Author: raul $
+ * @author $Author: mullan $
  */
 public final class XMLSignature extends SignatureElementProxy {
 
@@ -197,6 +197,15 @@ private Element signatureValueElement;
 
       super(doc);
 
+      String xmlnsDsPrefix = 
+         getDefaultPrefixBindings(Constants.SignatureSpecNS);
+      if (xmlnsDsPrefix == null) {
+         this._constructionElement.setAttributeNS
+	    (Constants.NamespaceSpecNS, "xmlns", Constants.SignatureSpecNS);
+      } else {
+         this._constructionElement.setAttributeNS
+	    (Constants.NamespaceSpecNS, xmlnsDsPrefix, Constants.SignatureSpecNS);
+      }
       XMLUtils.addReturnToElement(this._constructionElement);
 
       this._baseURI = BaseURI;
@@ -229,6 +238,15 @@ private Element signatureValueElement;
 
       super(doc);
 
+      String xmlnsDsPrefix = 
+         getDefaultPrefixBindings(Constants.SignatureSpecNS);
+      if (xmlnsDsPrefix == null) {
+         this._constructionElement.setAttributeNS
+	    (Constants.NamespaceSpecNS, "xmlns", Constants.SignatureSpecNS);
+      } else {
+         this._constructionElement.setAttributeNS
+	    (Constants.NamespaceSpecNS, xmlnsDsPrefix, Constants.SignatureSpecNS);
+      }
       XMLUtils.addReturnToElement(this._constructionElement);
 
       this._baseURI = BaseURI;
@@ -305,7 +323,7 @@ private Element signatureValueElement;
     */
    public void setId(String Id) {
 
-      if ((this._state == MODE_SIGN) && (Id != null)) {
+      if ( (Id != null)) {
          this._constructionElement.setAttributeNS(null, Constants._ATT_ID, Id);
          IdResolver.registerElementById(this._constructionElement, Id);
       }
@@ -347,31 +365,28 @@ private Element signatureValueElement;
       }
    }
 
-   /**
-    * Base64 encodes and sets the bytes as the content of the SignatureValue
-    * Node.
-    *
-    * @param bytes bytes to be used by SignatureValue before Base64 encoding
-    */
-   private void setSignatureValueElement(byte[] bytes)
-   {
+    /**
+     * Base64 encodes and sets the bytes as the content of the SignatureValue
+     * Node.
+     *
+     * @param bytes bytes to be used by SignatureValue before Base64 encoding
+     */
+    private void setSignatureValueElement(byte[] bytes) {
 
-      if (this._state == MODE_SIGN) {
-    	 while (signatureValueElement.hasChildNodes()) {
-        	 signatureValueElement.removeChild(signatureValueElement.getFirstChild());
-         }
+        while (signatureValueElement.hasChildNodes()) {
+            signatureValueElement.removeChild
+		(signatureValueElement.getFirstChild());
+        }
 
-         String base64codedValue = Base64.encode(bytes);
+        String base64codedValue = Base64.encode(bytes);
 
-         if (base64codedValue.length() > 76) {
-            base64codedValue = "\n" + base64codedValue + "\n";
-         }
+        if (base64codedValue.length() > 76 && !XMLUtils.ignoreLineBreaks()) {
+	    base64codedValue = "\n" + base64codedValue + "\n";
+        }
 
-         Text t = this._doc.createTextNode(base64codedValue);
-
-         signatureValueElement.appendChild(t);
-      }
-   }
+        Text t = this._doc.createTextNode(base64codedValue);
+        signatureValueElement.appendChild(t);
+    }
 
    /**
     * Returns the KeyInfo child. If we are in signing mode and the KeyInfo
@@ -384,7 +399,7 @@ private Element signatureValueElement;
    public KeyInfo getKeyInfo() {
 
       // check to see if we are signing and if we have to create a keyinfo
-      if ((this._state == MODE_SIGN) && (this._keyInfo == null)) {
+      if ( (this._keyInfo == null)) {
 
          // create the KeyInfo
          this._keyInfo = new KeyInfo(this._doc);
@@ -400,8 +415,7 @@ private Element signatureValueElement;
                // add it before the object
                this._constructionElement.insertBefore(keyInfoElement,
                                                       firstObject);
-               this._constructionElement
-                  .insertBefore(this._doc.createTextNode("\n"), firstObject);
+	       XMLUtils.addReturnBeforeChild(this._constructionElement, firstObject);
             } else {
 
                // add it as the last element to the signature
@@ -424,17 +438,17 @@ private Element signatureValueElement;
    public void appendObject(ObjectContainer object)
            throws XMLSignatureException {
 
-      try {
-         if (this._state != MODE_SIGN) {
-            throw new XMLSignatureException(
-               "signature.operationOnlyBeforeSign");
-         }
+      //try {
+         //if (this._state != MODE_SIGN) {
+           // throw new XMLSignatureException(
+             //  "signature.operationOnlyBeforeSign");
+         //}
 
          this._constructionElement.appendChild(object.getElement());
          XMLUtils.addReturnToElement(this._constructionElement);
-      } catch (XMLSecurityException ex) {
-         throw new XMLSignatureException("empty", ex);
-      }
+      //} catch (XMLSecurityException ex) {
+        // throw new XMLSignatureException("empty", ex);
+      //}
    }
 
    /**
@@ -480,7 +494,7 @@ private Element signatureValueElement;
       }
 
       try {
-         if (this._state == MODE_SIGN) {
+        // if (this._state == MODE_SIGN) {
             //Create a SignatureAlgorithm object
         	SignedInfo si = this.getSignedInfo();
             SignatureAlgorithm sa = si.getSignatureAlgorithm();               
@@ -502,7 +516,7 @@ private Element signatureValueElement;
 
             // set them on the SignateValue element
             this.setSignatureValueElement(jcebytes);
-         }
+         //}
       } catch (CanonicalizationException ex) {
          throw new XMLSignatureException("empty", ex);
       } catch (InvalidCanonicalizerException ex) {
@@ -579,11 +593,7 @@ private Element signatureValueElement;
       // If _followManifestsDuringValidation is true it will do the same for
       // References inside a Manifest.
       try {
-    	  SignedInfo si=this.getSignedInfo();
-         if (!si.verify(this._followManifestsDuringValidation)) {
-            return false;
-         }
-
+         SignedInfo si=this.getSignedInfo();
          //create a SignatureAlgorithms from the SignatureMethod inside
          //SignedInfo. This is used to validate the signature.
          SignatureAlgorithm sa =si.getSignatureAlgorithm();               
@@ -600,20 +610,22 @@ private Element signatureValueElement;
          OutputStream bos=new UnsyncBufferedOutputStream(so);
          si.signInOctectStream(bos);
          try {
-			bos.close();
-		} catch (IOException e) {
-			//Imposible
-		}
-         
+		bos.close();
+	 } catch (IOException e) {
+		//Imposible
+	 }
+
          //retrieve the byte[] from the stored signature
          byte sigBytes[] = this.getSignatureValue();
 
-
          //Have SignatureAlgorithm sign the input bytes and compare them to the
          //bytes that were stored in the signature.
-         boolean verify = sa.verify(sigBytes);
+         if (!sa.verify(sigBytes)) {
+            log.warn("Signature verification failed.");
+	    return false;
+	 }
 
-         return verify;
+         return si.verify(this._followManifestsDuringValidation);
       } catch (XMLSecurityException ex) {
          throw new XMLSignatureException("empty", ex);
       } 
