@@ -47,6 +47,16 @@ import org.w3c.dom.Text;
  */
 public class XMLUtils {
 
+   private static boolean ignoreLineBreaks = false;
+   static {
+      try {
+         ignoreLineBreaks = Boolean.getBoolean
+	    ("org.apache.xml.security.ignoreLineBreaks");
+      } catch (Exception e) {
+         // ignore exceptions
+      }
+   }
+	
    /**
     * Constructor XMLUtils
     *
@@ -217,7 +227,6 @@ public class XMLUtils {
 
 
    static  String dsPrefix=null;
-   static String xmlnsDsPrefix=null;
    static Map namePrefixes=new HashMap();
    /**
     * Creates an Element in the XML Signature specification namespace.
@@ -233,15 +242,8 @@ public class XMLUtils {
          throw new RuntimeException("Document is null");
       }
      
-
       if ((dsPrefix == null) || (dsPrefix.length() == 0)) {
-         Element element = doc.createElementNS(Constants.SignatureSpecNS,
-                                               elementName);
-
-         element.setAttributeNS(Constants.NamespaceSpecNS, "xmlns",
-                                Constants.SignatureSpecNS);
-
-         return element;
+         return doc.createElementNS(Constants.SignatureSpecNS, elementName);
       } 
       String namePrefix=(String) namePrefixes.get(elementName);
       if (namePrefix==null) {
@@ -251,15 +253,9 @@ public class XMLUtils {
     	  namePrefix=tag.toString();
     	  namePrefixes.put(elementName,namePrefix);
       }
-         Element element = doc.createElementNS(Constants.SignatureSpecNS, namePrefix);         
-         element.setAttributeNS(Constants.NamespaceSpecNS, xmlnsDsPrefix,
-                                Constants.SignatureSpecNS);
-
-         return element;
-      
+      return doc.createElementNS(Constants.SignatureSpecNS, namePrefix);
    }
 
-
    /**
     * Returns true if the element is in XML Signature namespace and the local
     * name equals the supplied one.
@@ -270,17 +266,7 @@ public class XMLUtils {
     */
    public static boolean elementIsInSignatureSpace(Element element,
            String localName) {
-
-      if ((element == null) ||
-          Constants.SignatureSpecNS!=element.getNamespaceURI() ){
-         return false;
-      }
-
-      if (!element.getLocalName().equals(localName)) {
-         return false;
-      }
-
-      return true;
+      return ElementProxy.checker.isNamespaceElement(element, localName, Constants.SignatureSpecNS);
    }
 
    /**
@@ -293,18 +279,7 @@ public class XMLUtils {
     */
    public static boolean elementIsInEncryptionSpace(Element element,
            String localName) {
-
-      if ((element == null) || 
-            EncryptionConstants.EncryptionSpecNS!=element.getNamespaceURI() 
-          ){
-         return false;
-      }
-
-      if (!element.getLocalName().equals(localName)) {
-         return false;
-      }
-
-      return true;
+	   return ElementProxy.checker.isNamespaceElement(element, localName, EncryptionConstants.EncryptionSpecNS);
    }
 
    /**
@@ -365,32 +340,6 @@ public class XMLUtils {
     }
     
 
- 
-   /**
-    * Method createDSctx
-    *
-    * @param doc
-    * @param prefix
-    * @param namespace
-    * @return the element.
-    */
-   public static Element createDSctx(Document doc, String prefix,
-                                     String namespace) {
-
-      if ((prefix == null) || (prefix.trim().length() == 0)) {
-         throw new IllegalArgumentException("You must supply a prefix");
-      }
-
-      Element ctx = doc.createElementNS(null, "namespaceContext");
-
-      ctx.setAttributeNS(Constants.NamespaceSpecNS, "xmlns:" + prefix.trim(),
-                         namespace);
-
-      return ctx;
-   }
-
-
-
    /**
     * Method addReturnToElement
     *
@@ -398,9 +347,23 @@ public class XMLUtils {
     */
    public static void addReturnToElement(Element e) {
 
-      Document doc = e.getOwnerDocument();
+      if (!ignoreLineBreaks) {
+         Document doc = e.getOwnerDocument();
+         e.appendChild(doc.createTextNode("\n"));
+      }
+   }
 
-      e.appendChild(doc.createTextNode("\n"));
+   public static void addReturnToElement(Document doc, HelperNodeList nl) {
+      if (!ignoreLineBreaks) {
+         nl.appendChild(doc.createTextNode("\n"));
+      }
+   }
+
+   public static void addReturnBeforeChild(Element e, Node child) {
+      if (!ignoreLineBreaks) {
+         Document doc = e.getOwnerDocument();
+         e.insertBefore(doc.createTextNode("\n"), child);
+      }
    }
 
    /**
@@ -523,8 +486,7 @@ public class XMLUtils {
     */
    public static Element selectDsNode(Node sibling, String nodeName, int number) {
 	while (sibling!=null) {
-		if (nodeName.equals(sibling.getLocalName())
-				&& Constants.SignatureSpecNS==sibling.getNamespaceURI()) {
+		if (ElementProxy.checker.isNamespaceElement(sibling, nodeName, Constants.SignatureSpecNS )) {
 			if (number==0){
 				return (Element)sibling;
 			}
@@ -544,8 +506,7 @@ public class XMLUtils {
 
    public static Element selectXencNode(Node sibling, String nodeName, int number) {
 	while (sibling!=null) {
-		if (nodeName.equals(sibling.getLocalName())
-				&& EncryptionConstants.EncryptionSpecNS==sibling.getNamespaceURI()) {
+		if (ElementProxy.checker.isNamespaceElement(sibling, nodeName, EncryptionConstants.EncryptionSpecNS )) {
 			if (number==0){
 				return (Element)sibling;
 			}
@@ -603,8 +564,7 @@ public class XMLUtils {
     */
    public static Element selectNode(Node sibling, String uri,String nodeName, int number) {
 	while (sibling!=null) {
-		if (nodeName.equals(sibling.getLocalName())
-				&& uri==sibling.getNamespaceURI()) {
+		if (ElementProxy.checker.isNamespaceElement(sibling, nodeName, uri)) {
 			if (number==0){
 				return (Element)sibling;
 			}
@@ -635,8 +595,7 @@ public class XMLUtils {
     	int curr=0;
     	//List list=new ArrayList();
     	while (sibling!=null) {
-    		if (nodeName.equals(sibling.getLocalName())
-    				&& uri==sibling.getNamespaceURI()) {
+    		if (ElementProxy.checker.isNamespaceElement(sibling, nodeName, uri)) {
     			a[curr++]=(Element)sibling;
     			if (size<=curr) {
     				int cursize= size<<2;
@@ -705,4 +664,8 @@ public class XMLUtils {
          }
       }
    }
+
+    public static boolean ignoreLineBreaks() {
+        return ignoreLineBreaks;
+    }
 }
