@@ -25,7 +25,7 @@
  * ===========================================================================
  */
 /*
- * $Id: DOMXMLSignature.java 410581 2006-05-31 16:26:56 +0000 (Wed, 31 May 2006) mullan $
+ * $Id: DOMXMLSignature.java 656799 2008-05-15 19:23:15Z mullan $
  */
 package org.jcp.xml.dsig.internal.dom;
 
@@ -33,6 +33,7 @@ import javax.xml.crypto.*;
 import javax.xml.crypto.dom.*;
 import javax.xml.crypto.dsig.*;
 import javax.xml.crypto.dsig.dom.DOMSignContext;
+import javax.xml.crypto.dsig.dom.DOMValidateContext;
 import javax.xml.crypto.dsig.keyinfo.KeyInfo;
 
 import java.io.*;
@@ -196,9 +197,7 @@ public final class DOMXMLSignature extends DOMStructure
             (ownerDoc, "Signature", XMLSignature.XMLNS, dsPrefix);
 
         // append xmlns attribute
-        //XXX I think this is supposed to be automatically inserted when
-        //XXX serializing a DOM2 tree, but doesn't seem to work with JAXP/Xalan
-        if (dsPrefix == null) {
+        if (dsPrefix == null || dsPrefix.length() == 0) {
             sigElem.setAttributeNS
 		("http://www.w3.org/2000/xmlns/", "xmlns", XMLSignature.XMLNS);
         } else {
@@ -236,10 +235,23 @@ public final class DOMXMLSignature extends DOMStructure
 	    throw new NullPointerException("validateContext is null");
 	}
 
+        if (!(vc instanceof DOMValidateContext)) {
+            throw new ClassCastException
+                ("validateContext must be of type DOMValidateContext");
+        }
+
 	if (validated) {
 	    return validationStatus;
 	}
 
+        // validate the signature
+        boolean sigValidity = sv.validate(vc);
+        if (!sigValidity) {
+            validationStatus = false;
+            validated = true;
+            return validationStatus;
+        }
+
         // validate all References
         List refs = this.si.getReferences();
         boolean validateRefs = true;
@@ -261,14 +273,6 @@ public final class DOMXMLSignature extends DOMStructure
 	    return validationStatus;
         }
 
-	// validate the signature
-	boolean sigValidity = sv.validate(vc);
-	if (!sigValidity) {
-	    validationStatus = false;
-	    validated = true;
-	    return validationStatus;
-        }
-
 	// validate Manifests, if property set
         boolean validateMans = true;
 	if (Boolean.TRUE.equals(vc.getProperty
@@ -420,8 +424,7 @@ public final class DOMXMLSignature extends DOMStructure
     }
 
     public int hashCode() {
-	// uncomment when JDK 1.4 is required
-	// assert false : "hashCode not designed";
+	assert false : "hashCode not designed";
 	return 54;
     }
 
